@@ -2,12 +2,13 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 
+import json
 from typing import Any, Literal, Optional, Union
 
 from haystack.lazy_imports import LazyImport
 from haystack.utils.auth import Secret
 
-with LazyImport(message="Run 'pip install \"sentence-transformers>=4.1.0\"'") as sentence_transformers_import:
+with LazyImport(message="Run 'pip install \"sentence-transformers>=5.0.0\"'") as sentence_transformers_import:
     from sentence_transformers import SentenceTransformer
 
 with LazyImport(message="Run 'pip install \"pillow\"'") as pillow_import:
@@ -23,10 +24,12 @@ class _SentenceTransformersEmbeddingBackendFactory:
 
     @staticmethod
     def get_embedding_backend(  # pylint: disable=too-many-positional-arguments
+        *,
         model: str,
         device: Optional[str] = None,
         auth_token: Optional[Secret] = None,
         trust_remote_code: bool = False,
+        revision: Optional[str] = None,
         local_files_only: bool = False,
         truncate_dim: Optional[int] = None,
         model_kwargs: Optional[dict[str, Any]] = None,
@@ -34,15 +37,31 @@ class _SentenceTransformersEmbeddingBackendFactory:
         config_kwargs: Optional[dict[str, Any]] = None,
         backend: Literal["torch", "onnx", "openvino"] = "torch",
     ):
-        embedding_backend_id = f"{model}{device}{auth_token}{truncate_dim}{backend}"
+        cache_params = {
+            "model": model,
+            "device": device,
+            "auth_token": auth_token,
+            "trust_remote_code": trust_remote_code,
+            "revision": revision,
+            "local_files_only": local_files_only,
+            "truncate_dim": truncate_dim,
+            "model_kwargs": model_kwargs,
+            "tokenizer_kwargs": tokenizer_kwargs,
+            "config_kwargs": config_kwargs,
+            "backend": backend,
+        }
+
+        embedding_backend_id = json.dumps(cache_params, sort_keys=True, default=str)
 
         if embedding_backend_id in _SentenceTransformersEmbeddingBackendFactory._instances:
             return _SentenceTransformersEmbeddingBackendFactory._instances[embedding_backend_id]
+
         embedding_backend = _SentenceTransformersEmbeddingBackend(
             model=model,
             device=device,
             auth_token=auth_token,
             trust_remote_code=trust_remote_code,
+            revision=revision,
             local_files_only=local_files_only,
             truncate_dim=truncate_dim,
             model_kwargs=model_kwargs,
@@ -50,6 +69,7 @@ class _SentenceTransformersEmbeddingBackendFactory:
             config_kwargs=config_kwargs,
             backend=backend,
         )
+
         _SentenceTransformersEmbeddingBackendFactory._instances[embedding_backend_id] = embedding_backend
         return embedding_backend
 
@@ -61,10 +81,12 @@ class _SentenceTransformersEmbeddingBackend:
 
     def __init__(  # pylint: disable=too-many-positional-arguments
         self,
+        *,
         model: str,
         device: Optional[str] = None,
         auth_token: Optional[Secret] = None,
         trust_remote_code: bool = False,
+        revision: Optional[str] = None,
         local_files_only: bool = False,
         truncate_dim: Optional[int] = None,
         model_kwargs: Optional[dict[str, Any]] = None,
@@ -79,6 +101,7 @@ class _SentenceTransformersEmbeddingBackend:
             device=device,
             token=auth_token.resolve_value() if auth_token else None,
             trust_remote_code=trust_remote_code,
+            revision=revision,
             local_files_only=local_files_only,
             truncate_dim=truncate_dim,
             model_kwargs=model_kwargs,
